package org.gcube.application.geoportal.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.jayway.jsonpath.JsonPath;
import org.bson.Document;
import org.gcube.application.cms.serialization.Serialization;
import org.gcube.application.cms.tests.TokenSetter;
import org.gcube.application.cms.tests.model.concessioni.TestConcessioniModel;
import org.gcube.application.geoportal.common.model.JSONPathWrapper;
import org.gcube.application.geoportal.common.model.document.ProfiledDocument;
import org.gcube.application.geoportal.common.model.document.filesets.RegisteredFileSet;
import org.gcube.application.geoportal.common.model.document.lifecycle.LifecycleInformation;
import org.gcube.application.geoportal.common.model.rest.Configuration;
import org.gcube.application.geoportal.common.model.rest.QueryRequest;
import org.gcube.application.geoportal.common.model.rest.RegisterFileSetRequest;
import org.gcube.application.geoportal.common.model.rest.StepExecutionRequest;
import org.gcube.application.geoportal.common.rest.InterfaceConstants;
import org.gcube.application.geoportal.common.utils.FileSets;
import org.gcube.application.geoportal.common.utils.StorageUtils;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.io.File;
import java.util.Collections;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public abstract class AbstractProfiledDocumentsTests extends BasicServiceTestUnit{


	@Before
	public void setContext(){
		TokenSetter.set(scope);
	}


	protected abstract WebTarget baseTarget();



	@Test
	public void getAll() {
		System.out.println(baseTarget().request(MediaType.APPLICATION_JSON).get(List.class));
	}

	@Test
	public void getByID(){
		baseTarget().request(MediaType.APPLICATION_JSON).get(List.class).forEach(d ->{

			try {
				check(baseTarget().path((Serialization.convert(d,ProfiledDocument.class)).get_id())
						.request(MediaType.APPLICATION_JSON).get(),ProfiledDocument.class);
			} catch (Exception e) {
				e.printStackTrace(System.err);
				Assert.fail(e.getMessage());
			}
		});
	}

	@Test
	public void getConfiguration() {
		System.out.println(baseTarget().path(InterfaceConstants.Methods.CONFIGURATION_PATH).request(MediaType.APPLICATION_JSON).get(Configuration.class));
	}

	// Queries
	@Test
	public void query() throws JsonProcessingException {
		System.out.println(baseTarget().path(InterfaceConstants.Methods.QUERY_PATH).
				request(MediaType.APPLICATION_JSON).
				post(Entity.entity(Serialization.write(new QueryRequest()),MediaType.APPLICATION_JSON)));
	}

	// CREATE / edit / delete



	// @@@@@@@@@@@@@@@  Routines

	protected ProfiledDocument createNew(Document content) throws Exception {
		return check(baseTarget().request(MediaType.APPLICATION_JSON).
				post(Entity.entity(content, MediaType.APPLICATION_JSON)),ProfiledDocument.class);
	}

	protected ProfiledDocument update(String id, Document newContent)throws Exception {
		return check(baseTarget().path(id).request(MediaType.APPLICATION_JSON).
				put(Entity.entity(newContent, MediaType.APPLICATION_JSON)),ProfiledDocument.class);
	}

	protected ProfiledDocument upload(StorageUtils storage,
									String id,
									String parentPath,
									String fieldName,
									String fieldDefinitionPath,
									Document attributes,
									RegisterFileSetRequest.ClashOptions clashPolicy,
									String ...files) throws Exception {
		FileSets.RequestBuilder builder = FileSets.build(parentPath,fieldName,fieldDefinitionPath);

		builder.setClashPolicy(clashPolicy).setAttributes(attributes);

		for(String file:files)
			builder.add(storage.putOntoStorage(new File(TestConcessioniModel.getBaseFolder(),file),file));

		return check(baseTarget().path(InterfaceConstants.Methods.REGISTER_FILES_PATH).path(id).request(MediaType.APPLICATION_JSON).
				post(Entity.entity(Serialization.write(builder.getTheRequest()),
						MediaType.APPLICATION_JSON)),ProfiledDocument.class);
	}


	protected ProfiledDocument step(String id, StepExecutionRequest request) throws Exception {
		ProfiledDocument toReturn= check(baseTarget().
				path(InterfaceConstants.Methods.STEP).path(id).request(MediaType.APPLICATION_JSON).
				post(Entity.entity(Serialization.write(request),
						MediaType.APPLICATION_JSON)),ProfiledDocument.class);

		assertTrue(toReturn.getLifecycleInformation().getLastInvokedStep().equals(request.getStepID()));
		return toReturn;
	}

}
